{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DerivingStrategies #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE GADTs #-}
{-# LANGUAGE GeneralizedNewtypeDeriving #-}
{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE QuasiQuotes #-}
{-# LANGUAGE StandaloneDeriving #-}
{-# LANGUAGE TemplateHaskell #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE UndecidableInstances #-}

module Datarekisteri.Backend.Sql.Types where

import Data.ByteString (ByteString)
import Data.Text (Text)
import Database.Persist (Entity, Key, entityKey, PersistEntity)
import Database.Persist.Postgresql.JSON (Value)
import Database.Persist.Sql (fromSqlKey, toSqlKey)
import Database.Persist.TH (persistUpperCase, mkPersist, sqlSettings)

import Datarekisteri.Core.Types
import Datarekisteri.Backend.Types

mkPersist sqlSettings [persistUpperCase|
SqlUser sql=users
  registered Time
  passwordCrypt PasswordHash
  permissions Text
  accepted (Maybe Time)
  memberData Value sqltype=jsonb

  deriving (Show)

SqlEmail sql=emails
  uid SqlUserId
  email Email sqltype=varchar(320)
  vid (Maybe SqlEmailVerificationId) sql=verification

  UniqueUserVerified uid vid
  -- This enables using persistent functions to get unique verified emails. The real
  -- constraint is stricter and doesn't allow having more than one null and one non-null
  -- verification but it's too complicated for persistent to understand.

  UniqueEmail email
  UniqueVerification vid

SqlEmailVerification sql=emailVerifications
  secret Text sqltype=varchar(255)
  expires Time

  UniqueVerificationSecret secret

SqlKey sql=keys
  uid SqlUserId
  data ByteString
  expires (Maybe Time)
  uploaded Time
  comment Text
  isPrimaryEncryptionKey Bool

SqlToken sql=tokens
  uid SqlUserId
  name (Maybe Text)
  data Text
  comment Text
  issued Time
  expires (Maybe Time)
  permissions Text

  UniqueNameUid name uid
  UniqueData data
|]

entityToID :: EntityID a => Entity (DB a) -> a
entityToID = toID . entityKey

class PersistEntity (DB a) =>  EntityID a where
    type DB a
    toID :: Key (DB a) -> a
    fromID :: a -> Key (DB a)

instance EntityID UserID where
    type DB UserID = SqlUser
    toID = UserID . fromIntegral . fromSqlKey
    fromID (UserID x) = toSqlKey $ fromIntegral x

instance EntityID TokenID where
    type DB TokenID = SqlToken
    toID = TokenID . fromIntegral . fromSqlKey
    fromID (TokenID x) = toSqlKey $ fromIntegral x

instance EntityID KeyID where
    type DB KeyID = SqlKey
    toID = KeyID . fromIntegral . fromSqlKey
    fromID (KeyID x) = toSqlKey $ fromIntegral x
