{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE GeneralizedNewtypeDeriving #-}
{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE QuasiQuotes #-}
{-# LANGUAGE RankNTypes #-}
{-# LANGUAGE TemplateHaskell #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE ViewPatterns #-}

{-# LANGUAGE NoImplicitPrelude #-}

import Relude hiding (get)

import System.Directory (createDirectoryIfMissing)
import Yesod.Static (static, Static)
import Yesod (mkYesodDispatch, warp)
import Yesod.Auth (getAuth)

import Options.Applicative

import Datarekisteri.Frontend.Types
import Datarekisteri.Frontend.Handlers

mkYesodDispatch "DataIdClient" resourcesDataIdClient

main :: IO ()
main = do
    config <- readConfig
    static <- getStaticDir "/tmp/data-id"
    warp (configPort config) $ DataIdClient static config

readConfig :: IO Config
readConfig = execParser $ info (configOpts' <**> helper)
    (fullDesc <> progDesc "Serve datarekisteri HTTP client"
    <> header "Client HTTP server for datarekisteri")

configOpts' :: Parser Config
configOpts' = Config
    <$> option auto (long "port" <> short 'p' <> metavar "PORT" <> value 3000 <> help "Port to listen on")
    <*> strOption (long "approot" <> short 'r' <> metavar "URL" <> value "http://localhost:3000" <> help "External URL of the server path /")
    <*> strOption (long "server-url" <> short 's' <> metavar "URL" <> value "http://localhost:3100" <> help "Internal URL of the API server")
    -- TODO make port depend on the --port flag

getStaticDir :: FilePath -> IO Static
getStaticDir dir = createDirectoryIfMissing True dir >> static dir
